'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.injectExtractionFunctions = injectExtractionFunctions;

var _struct = require('./struct');

var FN_PREFIX = '___SV_INJECT';
var FN_PREFIX_IMPLICIT_GLOBAL = 'IG';
var FN_PREFIX_EXPLICIT_GLOBAL = 'EG';
var FN_SUFFIX_VALUE = 'VALUE';
var TMP_VAR_PREFIX = '____SV_TEMP_VAR';

/**
 * Create injection function and source for a file, category, declaration and result handler
 */
function createInjection(fileId, categoryPrefix, declaration, idx, declarationResultHandler) {
  var fnName = FN_PREFIX + '_' + fileId + '_' + categoryPrefix + '_' + declaration.declarationClean + '_' + idx;
  var tmpVariableName = TMP_VAR_PREFIX + '_' + fileId;

  var injectedFunction = function injectedFunction(sassValue) {
    var value = (0, _struct.createStructuredValue)(sassValue);
    declarationResultHandler(declaration, value, sassValue);
    return sassValue;
  };

  var injectedCode = '$' + fnName + ': ' + fnName + '(' + declaration.declaration + ');\n';

  var mixinDep = declaration.deps.mixin;
  var functionDep = declaration.deps.function;

  // Inject mixin and function invocations if necessary
  if (mixinDep) {
    var mixinInvocation = '\n    ' + declaration.declaration + ': null !default;\n    @if ' + declaration.declaration + ' == null {\n      @include ' + mixinDep.name + '(' + new Array(mixinDep.argsCount.required).fill('null').join(', ') + ');\n    }\n    ';
    injectedCode = '' + mixinInvocation + injectedCode + '\n';
  }

  if (functionDep) {
    var functionInvocation = '\n    ' + declaration.declaration + ': null !default;\n    @if ' + declaration.declaration + ' == null {\n      $' + tmpVariableName + ': ' + functionDep.name + '(' + new Array(functionDep.argsCount.required).fill('null').join(', ') + ');\n    }\n    ';
    injectedCode = '' + functionInvocation + injectedCode + '\n';
  }

  return { fnName: fnName, injectedFunction: injectedFunction, injectedCode: injectedCode };
}

/**
 * Create injection functions for extraction variable values
 * Returns injection sass source and the injected functions
 * Declaration result handlers will be called with the extracted value of each declaration
 * Provided file id will be used to ensure unique function names per file
 */
function injectExtractionFunctions(fileId, declarations, _ref) {
  var globalDeclarationResultHandler = _ref.globalDeclarationResultHandler;

  var injectedData = '';
  var injectedFunctions = {};

  // Create injections for implicit global variables
  declarations.implicitGlobals.forEach(function (declaration, idx) {
    var _createInjection = createInjection(fileId, FN_PREFIX_IMPLICIT_GLOBAL, declaration, idx, globalDeclarationResultHandler),
        fnName = _createInjection.fnName,
        injectedFunction = _createInjection.injectedFunction,
        injectedCode = _createInjection.injectedCode;

    injectedFunctions[fnName] = injectedFunction;
    injectedData += injectedCode;
  });

  // Create injections for explicit global variables
  declarations.explicitGlobals.forEach(function (declaration, idx) {
    var _createInjection2 = createInjection(fileId, FN_PREFIX_EXPLICIT_GLOBAL, declaration, idx, globalDeclarationResultHandler),
        fnName = _createInjection2.fnName,
        injectedFunction = _createInjection2.injectedFunction,
        injectedCode = _createInjection2.injectedCode;

    injectedFunctions[fnName] = injectedFunction;
    injectedData += injectedCode;
  });

  return { injectedData: injectedData, injectedFunctions: injectedFunctions };
}