'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.parseDeclarations = parseDeclarations;

var _scssParser = require('scss-parser');

var _queryAst = require('query-ast');

var _queryAst2 = _interopRequireDefault(_queryAst);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var SCOPE_IMPICIT = 'implicit';
var SCOPE_EXPLICIT = 'explicit';
var DEP_KEYWORDS = {
  mixin: 'mixin',
  function: 'function'
};

/**
 * Check if a declaration node has a '!' operator followed by a '<flag>' identifier in its value
 */
function declarationHasFlag($ast, node, flag) {
  return $ast(node).children('value').children('operator').filter(function (operator) {
    return operator.node.value === '!';
  }).nextAll('identifier') // nextAll to account for potential space tokens following operator
  .filter(function (identifier) {
    return identifier.node.value === flag;
  }).length() > 0;
}

/**
 * Check if a declaration node has a '!' operator followed by the 'default' identifier in its value
 */
function isExplicitGlobalDeclaration($ast, node) {
  return declarationHasFlag($ast, node, 'global');
}

/**
 * Check if a declaration node has a '!' operator followed by the 'global' identifier in its value
 */
function isDefaultDeclaration($ast, node) {
  return declarationHasFlag($ast, node, 'default');
}

/**
 * Parse the raw expression of a variable declaration excluding flags
 */
function parseExpression($ast, declaration) {
  var flagsReached = false;

  return (0, _scssParser.stringify)($ast(declaration).children('value').get(0)).trim();
}

function getDeclarationDeps($ast, declaration, scope) {
  if (scope !== SCOPE_EXPLICIT) {
    return {};
  }

  var atruleNode = $ast(declaration).hasParents('atrule');

  if (atruleNode.length() < 1) {
    return {};
  }

  atruleNode = atruleNode.parentsUntil('atrule').parent();

  var atKeywordNode = atruleNode.children('atkeyword');
  var atIdentifierNode = atruleNode.children('identifier');
  var argumentsNode = atruleNode.children('arguments');

  if (atKeywordNode.length() === 0 || atIdentifierNode.length() === 0) {
    return {};
  }

  var atKeyword = atKeywordNode.first().value();
  var atIdentifier = atIdentifierNode.first().value();

  // Count arguments to mixin/function @atrule
  var requiredArgsCount = argumentsNode.children('variable').length();
  var optionalArgsCount = argumentsNode.children('declaration').length();
  var totalArgsCount = requiredArgsCount + optionalArgsCount;

  if (!DEP_KEYWORDS[atKeyword]) {
    return {};
  }

  return _defineProperty({}, DEP_KEYWORDS[atKeyword], {
    name: atIdentifier,
    argsCount: {
      total: totalArgsCount,
      required: requiredArgsCount,
      optional: optionalArgsCount
    }
  });
}

/**
 * Parse declaration node into declaration object
 */
function parseDeclaration($ast, declaration, scope) {
  var variable = {};

  var propertyNode = $ast(declaration).children('property');

  variable.declarationClean = propertyNode.value();
  variable.position = propertyNode.get(0).start;

  variable.declaration = '$' + variable.declarationClean;

  variable.expression = parseExpression($ast, declaration);
  variable.flags = {
    default: isDefaultDeclaration($ast, declaration),
    global: isExplicitGlobalDeclaration($ast, declaration)
  };

  variable.deps = getDeclarationDeps($ast, declaration, scope);

  return variable;
}

/**
 * Parse variable declarations from a chunk of sass source
 */
function parseDeclarations(data) {
  var ast = (0, _scssParser.parse)(data);
  var $ast = (0, _queryAst2.default)(ast);

  var implicitGlobalDeclarations = $ast('declaration').hasParent('stylesheet');
  var explicitGlobalDeclarations = $ast('declaration').hasParent('block').filter(function (node) {
    return isExplicitGlobalDeclaration($ast, node);
  });

  var implicitGlobals = implicitGlobalDeclarations.map(function (declaration) {
    return parseDeclaration($ast, declaration, SCOPE_IMPICIT);
  });
  var explicitGlobals = explicitGlobalDeclarations.map(function (declaration) {
    return parseDeclaration($ast, declaration, SCOPE_EXPLICIT);
  });

  return { explicitGlobals: explicitGlobals, implicitGlobals: implicitGlobals };
}